#!/usr/bin/env python

import os
import sys

import gobject
import gtk
font_size_range = range(5,60)

class TextViewDemo(gtk.Window):
    def __init__(self, parent=None):
        # Create the toplevel window
        gtk.Window.__init__(self)
        self.connect('destroy', lambda *w: gtk.main_quit())

        self.set_title(self.__class__.__name__)
        self.set_default_size(450, 450)
        self.set_border_width(0)

        # For convenience, we just use the autocreated buffer from
        # the first text view; you could also create the buffer
        # by itself with gtk.text_buffer_new(), then later create
        # a view widget.

        view1 = gtk.TextView();
        buffer_1 = view1.get_buffer()
        view2 = gtk.TextView(buffer_1)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(view1)
        self.add(sw)

        self.create_tags(buffer_1)
        self.insert_text(buffer_1)
        self.win = None
        self.show_all()

    def create_tags(self, text_buffer):
        '''
        Create a bunch of tags. Note that it's also possible to
        create tags with gtk.text_tag_new() then add them to the
        tag table for the buffer, text_buffer.create_tag() is
        just a convenience function. Also note that you don't have
        to give tags a name; pass None for the name to create an
        anonymous tag.

        In any real app, another useful optimization would be to create
        a GtkTextTagTable in advance, and reuse the same tag table for
        all the buffers with the same tag set, instead of creating
        new copies of the same tags for every buffer.

        Tags are assigned default priorities in order of addition to the
        tag table. That is, tags created later that affect the same text
        property affected by an earlier tag will override the earlier
        tag. You can modify tag priorities with
        gtk.text_tag_set_priority().
        '''

        import pango
        text_buffer.create_tag("heading",
                    weight=pango.WEIGHT_BOLD,
                    size=15 * pango.SCALE)

        text_buffer.create_tag("italic", style=pango.STYLE_ITALIC)

        text_buffer.create_tag("bold", weight=pango.WEIGHT_BOLD)

                                # points times the pango.SCALE factor
        text_buffer.create_tag("big", size=20 * pango.SCALE)

        text_buffer.create_tag("xx-small", scale=pango.SCALE_XX_SMALL)

        text_buffer.create_tag("x-large", scale=pango.SCALE_X_LARGE)

        text_buffer.create_tag("monospace", family="monospace")
        text_buffer.create_tag("times", font="Times New Roman, 20")
        for size in font_size_range:
            text_buffer.create_tag("size-%d"%size, size=size * pango.SCALE)
        for size in font_size_range:
            text_buffer.create_tag("times-%d"%size, font="Times New Roman, %d"%size)
        for size in font_size_range:
            text_buffer.create_tag("arial-%d"%size, font="Arial, %d"%size)

        text_buffer.create_tag("blue_foreground", foreground="blue")

        text_buffer.create_tag("red_background", background="red")

        text_buffer.create_tag("big_gap_before_line", pixels_above_lines=30)

        text_buffer.create_tag("big_gap_after_line", pixels_below_lines=30)

        text_buffer.create_tag("double_spaced_line", pixels_inside_wrap=10)

        text_buffer.create_tag("not_editable", editable=False)

        text_buffer.create_tag("word_wrap", wrap_mode=gtk.WRAP_WORD)

        text_buffer.create_tag("char_wrap", wrap_mode=gtk.WRAP_CHAR)

        text_buffer.create_tag("no_wrap", wrap_mode=gtk.WRAP_NONE)

        text_buffer.create_tag("center", justification=gtk.JUSTIFY_CENTER)

        text_buffer.create_tag("right_justify", justification=gtk.JUSTIFY_RIGHT)

        text_buffer.create_tag("wide_margins",
                    left_margin=50, right_margin=50)

        text_buffer.create_tag("strikethrough", strikethrough=True)

        text_buffer.create_tag("underline",
                    underline=pango.UNDERLINE_SINGLE)

        text_buffer.create_tag("double_underline",
                    underline=pango.UNDERLINE_DOUBLE)

        text_buffer.create_tag("superscript",
                    rise=10 * pango.SCALE,      # 10 pixels
                    size=8 * pango.SCALE)       #  8 points

        text_buffer.create_tag("subscript",
                    rise=-10 * pango.SCALE,     # 10 pixels
                    size=8 * pango.SCALE)       #  8 points

        text_buffer.create_tag("rtl_quote",
                    wrap_mode=gtk.WRAP_WORD, direction=gtk.TEXT_DIR_RTL,
                    indent=30, left_margin=20, right_margin=20)

    def insert_text(self, text_buffer):
        iter = text_buffer.get_iter_at_offset(0)

        
        text_buffer.insert_with_tags_by_name(iter,
            "Internationalization. ", "heading")

        text_buffer.insert(iter,
            "You can put all sorts of Unicode text in the buffer.\n\n"
            "German(Deutsch S\303\274d) Gr\303\274\303\237 Gott\nGreek"
            "(\316\225\316\273\316\273\316\267\316\275\316\271\316\272"
            "\316\254) \316\223\316\265\316\271\316\254 \317\203\316\261"
            "\317\202\nHebrew   \327\251\327\234\327\225\327\235\n"
            "Japanese(\346\227\245\346\234\254\350\252\236)\n\nThe "
            "widget properly handles bidirectional text, word wrapping, "
            "DOS/UNIX/Unicode paragraph separators, grapheme boundaries, "
            "and so on using the Pango internationalization framework.\n", -1)
        
        jp = "\346\227\245\346\234\254\350\252\236f"
        for i in range(200):
            text_buffer.insert(iter,(jp*20)+'\n', -1)
        
        for i in range(100):
            text_buffer.insert_with_tags_by_name(iter,
                "\331\210\331\202\330\257 \330\250\330\257\330\243 "
                "\330\253\331\204\330\247\330\253 \331\205\331\206 "
                "\330\243\331\203\330\253\330\261 \330\247\331\204\331"
                "\205\330\244\330\263\330\263\330\247\330\252 \330\252"
                "\331\202\330\257\331\205\330\247 \331\201\331\212 \330"
                "\264\330\250\331\203\330\251 \330\247\331\203\330\263"
                "\331\212\331\210\331\206 \330\250\330\261\330\247\331"
                "\205\330\254\331\207\330\247 \331\203\331\205\331\206"
                "\330\270\331\205\330\247\330\252 \331\204\330\247 \330"
                "\252\330\263\330\271\331\211 \331\204\331\204\330\261"
                "\330\250\330\255\330\214 \330\253\331\205 \330\252\330"
                "\255\331\210\331\204\330\252 \331\201\331\212 \330\247"
                "\331\204\330\263\331\206\331\210\330\247\330\252 \330"
                "\247\331\204\330\256\331\205\330\263 \330\247\331\204"
                "\331\205\330\247\330\266\331\212\330\251 \330\245\331"
                "\204\331\211 \331\205\330\244\330\263\330\263\330\247"
                "\330\252 \331\205\330\247\331\204\331\212\330\251 \331"
                "\205\331\206\330\270\331\205\330\251\330\214 \331\210"
                "\330\250\330\247\330\252\330\252 \330\254\330\262\330\241"
                "\330\247 \331\205\331\206 \330\247\331\204\331\206\330\270"
                "\330\247\331\205 \330\247\331\204\331\205\330\247\331\204"
                "\331\212 \331\201\331\212 \330\250\331\204\330\257\330\247"
                "\331\206\331\207\330\247\330\214 \331\210\331\204\331\203"
                "\331\206\331\207\330\247 \330\252\330\252\330\256\330\265"
                "\330\265 \331\201\331\212 \330\256\330\257\331\205\330\251 "
                "\331\202\330\267\330\247\330\271 \330\247\331\204\331\205\330"
                "\264\330\261\331\210\330\271\330\247\330\252 \330\247\331\204"
                "\330\265\330\272\331\212\330\261\330\251. \331\210\330\243"
                "\330\255\330\257 \330\243\331\203\330\253\330\261 \331\207"
                "\330\260\331\207 \330\247\331\204\331\205\330\244\330\263"
                "\330\263\330\247\330\252 \331\206\330\254\330\247\330\255"
                "\330\247 \331\207\331\210 \302\273\330\250\330\247\331\206"
                "\331\203\331\210\330\263\331\210\331\204\302\253 \331\201"
                "\331\212 \330\250\331\210\331\204\331\212\331\201\331\212"
                "\330\247.\n\n", "rtl_quote")


            text_buffer.insert_with_tags_by_name(iter,
                "\331\210\331\202\330\257 \330\250\330\257\330\243 "
                "\330\253\331\204\330\247\330\253 \331\205\331\206 "
                "\330\243\331\203\330\253\330\261 \330\247\331\204\331"
                "\205\330\244\330\263\330\263\330\247\330\252 \330\252"
                "\331\202\330\257\331\205\330\247 \331\201\331\212 \330"
                "\264\330\250\331\203\330\251 \330\247\331\203\330\263"
                "\331\212\331\210\331\206 \330\250\330\261\330\247\331"
                "\205\330\254\331\207\330\247 \331\203\331\205\331\206", "times")

        arabic =  '\331\210\331\202\330\257 \n'
        for i in font_size_range+list(reversed(font_size_range)):

            text_buffer.insert_with_tags_by_name(iter,
                'times: '+arabic, "times-%d"%i)
            text_buffer.insert_with_tags_by_name(iter,
              'arial: '+arabic , "arial-%d"%i)
            text_buffer.insert_with_tags_by_name(iter,
              jp+'\n' , "size-%d"%i)

        # Apply word_wrap tag to whole buffer */
        start, end = text_buffer.get_bounds()
        text_buffer.apply_tag_by_name("word_wrap", start, end)

def main():
    TextViewDemo()
    gtk.main()

if __name__ == '__main__':
    main()
